// Copyright (c) 2016, teja. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library jaguar_orm_sqljocky.src;

import 'package:jaguar_query/jaguar_query.dart';
import 'package:sqljocky2/sqljocky.dart' as sj;
import 'package:sqljocky2/src/query/standard_data_packet.dart' as sj;
import 'dart:async';

abstract class JaguarOrmException {}

class NoRecordFound implements JaguarOrmException {
  const NoRecordFound();

  String toString() => 'No record found!';
}

class SqlJockyAdapter implements Adapter<sj.ConnectionPool> {
  sj.ConnectionPool _connection;

  final String host, username, password, databaseName;
  final int port;

  SqlJockyAdapter(this.databaseName,
      {this.username, this.password, this.host: 'localhost', this.port: 3306});

  SqlJockyAdapter.FromConnection(this._connection)
      : host = null,
        username = null,
        password = null,
        databaseName = null,
        port = null;

  /// Connects to the database
  Future connect() async {
    if(_connection != null) return;
    _connection = new sj.ConnectionPool(
        host: host,
        port: port,
        user: username,
        password: password,
        db: databaseName,
        max: 5);
  }

  /// Closes all connections to the database.
  Future close() async {
    _connection.closeConnectionsWhenNotInUse();
  }

  sj.ConnectionPool get connection => _connection;

  /// Finds one record in the table
  Future<Map> findOne(FindStatement statement) async {
    sj.Results stream = await _connection.query(statement.toSql());

    sj.StandardDataPacket rowFound;
    await for (sj.StandardDataPacket row in stream) {
      rowFound = row;
      break;
    }

    if (rowFound == null) {
      throw const NoRecordFound();
    }

    return rowFound.fields;
  }

  // Finds many records in the table
  Future<Stream<Map>> find(FindStatement statement) async {
    StreamTransformer transformer = new StreamTransformer.fromHandlers(
        handleData: (sj.StandardDataPacket row, EventSink<Map> sink) {
      sink.add(row.fields);
    });

    sj.Results stream = await _connection.query(statement.toSql());
    return stream.transform(transformer);
  }

  /// Inserts a record into the table
  Future<dynamic> insert(InsertStatement st) async {
    sj.Results result = await _connection.query(st.toSql());

    print(result);

    return null;
  }

  /// Updates a record in the table
  Future update(UpdateStatement st) async {
    await _connection.query(st.toSql());
  }

  /// Deletes a record from the table
  Future delete(DeleteStatement st) async {
    await _connection.query(st.toSql());
  }
}
